###############   PSQF 7375 Advanced Longitudinal Example 4a using R   #############

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC in lmer

if (!require("prediction")) install.packages("prediction")
library(prediction) # To get predicted outcomes like in STATA margins

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

###############################################################################
# Jonathan's function to add level-two means to values
addUnitMeans = function(data, unitVariable, meanVariables, newNames){
  # create unit-level data set:
  unitMeans = t(sapply(
    X = unique(data[, unitVariable]),
    FUN = function(x, data) {
      return(c(
        x,
        length(which(data[, unitVariable] == x)),
        apply(
          X = as.data.frame(data[which(data[, unitVariable] == x), meanVariables]),
          MARGIN = 2,
          FUN = mean,
          rm.na = TRUE
        )
      ))
    },
    data = data
  ))
  unitMeans = data.frame(unitMeans)
  names(unitMeans) = c(unitVariable, paste0("Nper", unitVariable), newNames)
  newData = merge(x = data, y = unitMeans, by = unitVariable)
  return(newData)
}

################################################################################
#####   BEGIN DATA MANIPULATION OF EXAMPLE 4a TIME-VARYING PREDICTORS      #####
#####              CHANGE "filesave" TO YOUR DIRECTORY                     #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF7375_AdvLong/PSQF7375_AdvLong_Example4a/"
filename = "AdvLong_Example4a.xlsx"
setwd(dir=filesave)

# Import Example4a stacked data from excel in sheet "Data"
Example4a = read_excel(paste0(filesave,filename), sheet="Example4a") 
# Convert to data frame to use in analysis
Example4a = as.data.frame(Example4a)

# Make a person mean for mood by averaging across rows for same ID
# Uses function from above to add person means (to same data here)
Example4a = addUnitMeans(data=Example4a, unitVariable="ID",
            meanVariables=c("negmood"), newNames=c("PMnegmood"))

# Center level-2 person mean of negative mood (uncentered because mean=0, SD=1)
Example4a$PMnm0 = Example4a$PMnegmood-0
# Make level-1 predictor to use with PERSON-MEAN-CENTERING (0=PM)
Example4a$WPnm = Example4a$negmood-Example4a$PMnegmood
# Make level-1 predictor to use with GRAND-MEAN-CENTERING (0=0)
Example4a$TVnm0 = Example4a$negmood-0
# Binary gender already exists as sexMW

# Open output text file
txtStart(file=paste0(filesave,"AdvLong_Example4a_R_Output.txt"))


################################################################################
#####               PART 1: VARIANCE PARTITIONING                          #####
################################################################################

print("R Model 1a: Empty Model for Daily Glucose Outcome")
Empty = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
llikAIC(Empty, chkREML=FALSE); summary(Empty, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT"); icc(Empty); ranova(Empty)

txtComment(" ") # insert blank space in output


print("R Model 1b: Empty Model for Daily Negative Mood Predictor")
EmptyMood = lmer(data=Example4a, REML=TRUE, formula=negmood~1+(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(EmptyMood, chkREML=FALSE); summary(EmptyMood, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT"); icc(EmptyMood); ranova(EmptyMood)

txtComment(" ") # insert blank space in output


################################################################################
#####       PART 2: PERSON-MEAN-CENTERING (VARIABLE-CENTERING)             #####
################################################################################

print("R Model 2a: Fixed Effects of Negative Mood using Person-MC")
FixWP = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+WPnm+PMnm0+(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(FixWP, chkREML=FALSE); summary(FixWP, ddf="Satterthwaite") 
print("F-Test for 2 Mood Slopes")
contestMD(FixWP, ddf="Satterthwaite", L=rbind(c(0,1,0),c(0,0,1)))
print("L1 Within-Person Mood Effect");  contest1D(FixWP, L=c(0, 1,0))
print("L2 Between-Person Mood Effect"); contest1D(FixWP, L=c(0, 0,1))
print("L2 Contextual Mood Effect");     contest1D(FixWP, L=c(0,-1,1))
# Save predicted outcomes to compute total-R2
Example4a$PredMoodP = predict(FixWP, re.form=NA)
rFixWP = cor.test(Example4a$PredMoodP, Example4a$lglucAM, method="pearson")
print("Total R2"); rFixWP$estimate^2


txtComment(" ") # insert blank space in output


print("R Model 2b: Random Effect of WP Negative Mood using Person-MC")
RandWP = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+WPnm+PMnm0+(1+WPnm|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance"); 
llikAIC(RandWP, chkREML=FALSE); summary(RandWP, ddf="Satterthwaite") 
print("LRT against fixed effect model"); ranova(RandWP)

txtComment(" ") # insert blank space in output


print("R Model 2c: Fixed Effects of Sex (0=M, 1=W) by Person-MC Negative Mood")
SexWP = lmer(data=Example4a, REML=TRUE, 
             formula=lglucAM~1+WPnm+PMnm0+sexMW +WPnm:sexMW +PMnm0:sexMW +(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(SexWP, chkREML=FALSE); summary(SexWP, ddf="Satterthwaite") 
print("F-Test for 3 Sex Slopes")
contestMD(SexWP, ddf="Satterthwaite", 
          L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))
print("Intercept: Men (at mood=0)");                contest1D(SexWP, L=c(1, 0,0,0, 0,0))
print("Intercept: Women (at mood=0)");              contest1D(SexWP, L=c(1, 0,0,1, 0,0))
print("Intercept: Women Diff (at mood=0)");         contest1D(SexWP, L=c(0, 0,0,1, 0,0))
print("L1 Within-Person Mood Effect: Men");         contest1D(SexWP, L=c(0, 1,0,0, 0,0))
print("L1 Within-Person Mood Effect: Women");       contest1D(SexWP, L=c(0, 1,0,0, 1,0))
print("L1 Within-Person Mood Effect: Women Diff");  contest1D(SexWP, L=c(0, 0,0,0, 1,0))
print("L2 Between-Person Mood Effect: Men");        contest1D(SexWP, L=c(0, 0,1,0, 0,0))
print("L2 Between-Person Mood Effect: Women");      contest1D(SexWP, L=c(0, 0,1,0, 0,1))
print("L2 Between-Person Mood Effect: Women Diff"); contest1D(SexWP, L=c(0, 0,0,0, 0,1))
print("L2 Contextual Mood Effect: Men");            contest1D(SexWP, L=c(0,-1,1,0, 0,0))
print("L2 Contextual Mood Effect: Women");          contest1D(SexWP, L=c(0,-1,1,0,-1,1))
print("L2 Contextual Mood Effect: Women Diff");     contest1D(SexWP, L=c(0, 0,0,0,-1,1))
# Save predicted outcomes to compute total-R2
Example4a$PredSexP = predict(SexWP, re.form=NA)
rSexWP = cor.test(Example4a$PredSexP, Example4a$lglucAM, method="pearson")
print("Total R2"); rSexWP$estimate^2
print("Change in Total R2"); rSexWP$estimate^2-rFixWP$estimate^2
print("Predicted Outcomes for Men")
PredMenP = summary(prediction(model=SexWP, type="response",
           at=list(WPnm=seq(-1,1,by=1), PMnm0=seq(-1,1,by=2), sexMW=0))); PredMenP
print("Predicted Outcomes for Women")
PredWomenP = summary(prediction(model=SexWP, type="response",
             at=list(WPnm=seq(-1,1,by=1), PMnm0=seq(-1,1,by=2), sexMW=1))); PredWomenP

txtComment(" ") # insert blank space in output


################################################################################
#####        PART 3: GRAND-MEAN-CENTERING (VARIABLE-CENTERING)             #####
################################################################################

print("R Smushed Model 3: Fixed Effect of Negative Mood only using Grand-MC")
Smush = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+TVnm0+(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
llikAIC(Smush, chkREML=FALSE); summary(Smush, ddf="Satterthwaite")

txtComment(" ") # insert blank space in output


print("R Model 3a: Fixed Effects of Negative Mood using Person-MC")
FixTV = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+TVnm0+PMnm0+(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance"); 
llikAIC(FixTV, chkREML=FALSE); summary(FixTV, ddf="Satterthwaite")
print("F-Test fof 2 Mood Slopes")
contestMD(FixTV, ddf="Satterthwaite", L=rbind(c(0,1,0),c(0,0,1)))
print("L1 Within-Person Mood Effect");  contest1D(FixTV, L=c(0,1,0))
print("L2 Between-Person Mood Effect"); contest1D(FixTV, L=c(0,1,1))
print("L2 Contextual Mood Effect");     contest1D(FixTV, L=c(0,0,1))
# Save predicted outcomes to compute total-R2
Example4a$PredMoodG = predict(FixTV, re.form=NA)
rFixTV = cor.test(Example4a$PredMoodG, Example4a$lglucAM, method="pearson")
print("Total R2"); rFixTV$estimate^2

txtComment(" ") # insert blank space in output


print("R Model 3b: Random Effect of TV Negative Mood using Grand-MC")
print("FOR COMPARISON WITH MODEL 2B ONLY BECAUSE RANDOM SLOPE IS SMUSHED")
RandTV = lmer(data=Example4a, REML=TRUE, formula=lglucAM~1+TVnm0+PMnm0+(1+TVnm0|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance"); 
llikAIC(RandTV, chkREML=FALSE); summary(RandTV, ddf="Satterthwaite")
print("LRT against fixed effect model"); ranova(RandTV)

txtComment(" ") # insert blank space in output


print("R Model 3c: Fixed Effects of Sex (0=M, 1=W) by Grand-MC Negative Mood")
SexTV = lmer(data=Example4a, REML=TRUE, 
             formula=lglucAM~1+TVnm0+PMnm0+sexMW +TVnm0:sexMW +PMnm0:sexMW +(1|ID))
print("Show results using Satterthwaite DDF including -2LL as deviance"); 
llikAIC(SexTV, chkREML=FALSE); summary(SexTV, ddf="Satterthwaite")
print("F-Test fof 3 Sex Slopes")
contestMD(SexTV, ddf="Satterthwaite", 
          L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))
print("Intercept: Men (at mood=0)");                contest1D(SexTV, L=c(1,0,0,0,0,0))
print("Intercept: Women (at mood=0)");              contest1D(SexTV, L=c(1,0,0,1,0,0))
print("Intercept: Women Diff (at mood=0)");         contest1D(SexTV, L=c(0,0,0,1,0,0))
print("L1 Within-Person Mood Effect: Men");         contest1D(SexTV, L=c(0,1,0,0,0,0))
print("L1 Within-Person Mood Effect: Women");       contest1D(SexTV, L=c(0,1,0,0,1,0))
print("L1 Within-Person Mood Effect: Women Diff");  contest1D(SexTV, L=c(0,0,0,0,1,0))
print("L2 Between-Person Mood Effect: Men");        contest1D(SexTV, L=c(0,1,1,0,0,0))
print("L2 Between-Person Mood Effect: Women");      contest1D(SexTV, L=c(0,1,1,0,1,1))
print("L2 Between-Person Mood Effect: Women Diff"); contest1D(SexTV, L=c(0,0,0,0,1,1))
print("L2 Contextual Mood Effect: Men");            contest1D(SexTV, L=c(0,0,1,0,0,0))
print("L2 Contextual Mood Effect: Women");          contest1D(SexTV, L=c(0,0,1,0,0,1))
print("L2 Contextual Mood Effect: Women Diff");     contest1D(SexTV, L=c(0,0,0,0,0,1))
# Save predicted outcomes to compute total-R2
Example4a$PredSexG = predict(SexTV, re.form=NA)
rSexTV = cor.test(Example4a$PredSexG, Example4a$lglucAM, method="pearson")
print("Total R2"); rSexTV$estimate^2
print("Change in Total R2"); rSexTV$estimate^2-rFixTV$estimate^2
print("Predicted Outcomes for Men")
PredMenG = summary(prediction(model=SexTV, type="response",
           at=list(TVnm0=seq(-2,2,by=1), PMnm0=seq(-1,1,by=2), sexMW=0))); PredMenG
print("Predicted Outcomes for Women")
PredWomenG = summary(prediction(model=SexTV, type="response",
             at=list(TVnm0=seq(-2,2,by=1), PMnm0=seq(-1,1,by=2), sexMW=1))); PredWomenG

txtComment(" ") # insert blank space in output


###########################################################################################
# Close external file of saved results
txtStop()



