// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

*********************************************************************************
******  BEGIN DATA MANIPULATION FOR EXAMPLE 4A TIME-VARYING PREDICTORS    *******
******               CHANGE "filesave" to your directory                  *******
*********************************************************************************

// Defining working directory for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
cd "C:\Dropbox\23_PSQF7375_AdvLong\PSQF7375_AdvLong_Example4a"

// Import Example4a long data
clear // clear memory in case a dataset is already open
import excel "AdvLong_Example4a.xlsx", firstrow case(preserve) clear 

// Sort by ID, make a person mean for mood by averaging across rows for same ID
sort ID
egen PMnegmood = mean(negmood), by(ID)
// Center level-2 person mean of negative mood (uncentered because mean=0, SD=1))
gen PMnm0 = PMnegmood - 0
label variable PMnm0 "PMnm0: Person Mean Negative Mood (0=0)"

// Make level-1 predictor to use with PERSON-MEAN-CENTERING
gen WPnm = negmood - PMnegmood
label variable WPnm "WPnm: Within-Person Negative Mood (0=PM)"
// Make level-1 predictor to use with GRAND-MEAN-CENTERING
gen TVnm0 = negmood - 0
label variable TVnm0 "TVnm0: Time-Varying Negative Mood (0=0)"

// Binary gender already exists
label variable sexMW "sexMW: Participant Sex (0=M, 1=W)"

// Open external file to save results to
log using AdvLong_Example4a_STATA_Output.log, replace


******************************************************************************
******                  PART 1: VARIANCE PARTITIONING                   ******
******************************************************************************

display "STATA Model 1a: Empty Model for Daily Glucose Outcome"
mixed lglucAM  , || ID: , ///  
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2    // Print -2LL for model
estat icc                     // Intraclass correlation 

display "STATA Model 1b: Empty Model for Daily Negative Mood Predictor"
mixed negmood  , || ID: , ///  
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2    // Print -2LL for model
estat icc                     // Intraclass correlation  


******************************************************************************
******        PART 2: PERSON-MEAN-CENTERING (VARIABLE-CENTERING)        ******
******************************************************************************
    
display "STATA Model 2a: Fixed Effects of Negative Mood using Person-MC"
mixed lglucAM c.WPnm c.PMnm0, || ID: , ///  
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2         // Print -2LL for model 
estimates store FixWP              // Save LL for LRT
test (c.WPnm=0)(c.PMnm0=0), small  // F-test of 2 Mood Slopes
lincom c.WPnm*1, small                // L1 Within-Person Mood Effect
lincom c.PMnm0*1, small               // L2 Between-Person Mood Effect
lincom c.WPnm*-1 + c.PMnm0*1, small   // L2 Contextual Mood Effect
predict predmoodP        // Save fixed-effect predicted outcomes
corr lglucAM predmoodP   // Get total r to make R2
     display r(rho)^2    // Print total R2 relative to empty model
    
display "STATA Model 2b: Random Effect of WP Negative Mood using Person-MC"
mixed lglucAM c.WPnm c.PMnm0, || ID: WPnm, covariance(un) /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2         // Print -2LL for model
estat recovariance, relevel(ID) correlation  // GCORR matrix 
estimates store RandWP             // Save LL for LRT
lrtest RandWP FixWP                // LRT against fixed WPnm model

display "STATA Model 2c: Fixed Effects of Sex (0=M, 1=W) by Person-MC Negative Mood" 
mixed lglucAM c.WPnm c.PMnm0 c.sexMW c.WPnm#c.sexMW c.PMnm0#c.sexMW, || ID: , /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model
test (c.sexMW=0)(c.WPnm#c.sexMW=0)(c.PMnm0#c.sexMW=0), small  // F-test of 3 Sex Slopes
lincom _cons*1 + c.sexMW*0, small         // Intercept: Men (at mood=0)
lincom _cons*1 + c.sexMW*1, small         // Intercept: Women (at mood=0)
lincom c.sexMW*1, small                   // Intercept: Women Diff (at mood=0)
lincom c.WPnm*1 + c.WPnm#c.sexMW*0, small    // L1 Within-Person Mood Effect: Men
lincom c.WPnm*1 + c.WPnm#c.sexMW*1, small    // L1 Within-Person Mood Effect: Women
lincom c.WPnm#c.sexMW*1, small               // L1 Within-Person Mood Effect: Women diff
lincom c.PMnm0*1 + c.PMnm0#c.sexMW*0, small     // L2 Between-Person Mood Effect: Men
lincom c.PMnm0*1 + c.PMnm0#c.sexMW*1, small     // L2 Between-Person Mood Effect: Women
lincom c.PMnm0#c.sexMW*1, small                 // L2 Between-Person Mood Effect: Women Diff
lincom c.WPnm*-1 + c.WPnm#c.sexMW*0  + c.PMnm0*1 + c.PMnm0#c.sexMW*0, small  // L2 Contextual Mood Effect: Men
lincom c.WPnm*-1 + c.WPnm#c.sexMW*-1 + c.PMnm0*1 + c.PMnm0#c.sexMW*1, small  // L2 Contextual Mood Effect: Women
lincom c.WPnm#c.sexMW*-1 + c.PMnm0#c.sexMW*1, small                          // L2 Contextual Mood Effect: Women Diff 
margins, at(c.WPnm=(-1 0 1) c.PMnm0=(-1 1) c.sexMW=(0 1)) vsquish   // Create predicted values
marginsplot, noci xdimension(WPnm) name(predicted_meansP, replace)  // Plot predicted values, no CI
graph export "STATA Sex Moderation Plot WPnm.png", replace
predict predsexP        // Save fixed-effect predicted outcomes
corr lglucAM predsexP   // Get total r to make R2
     display r(rho)^2   // Print total R2 relative to empty model 
        
            
******************************************************************************
******        PART 3: GRAND-MEAN-CENTERING (CONSTANT-CENTERING)         ******
******************************************************************************

display "STATA Smushed Model 3: Fixed Effect of TV Negative Mood only using Grand-MC"
mixed lglucAM c.TVnm0, || ID: , /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog 
display "-2LL = " e(ll)*-2    // Print -2LL for model
    
display "STATA Model 3a: Fixed Effects of Negative Mood using GMC"
mixed lglucAM c.TVnm0 c.PMnm0, || ID: , /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2          // Print -2LL for model
estimates store FixTV               // Save LL for LRT
test (c.TVnm0=0)(c.PMnm0=0), small  // F-test of 2 Mood Slopes
lincom c.TVnm0*1, small                // L1 Within-Person Mood Effect
lincom c.TVnm0*1 + c.PMnm0*1, small    // L2 Between-Person Mood Effect
lincom c.PMnm0*1, small                // L2 Contextual Mood Effect
predict predmoodG        // Save fixed-effect predicted outcomes
corr lglucAM predmoodG   // Get total r to make R2
     display r(rho)^2    // Print total R2 relative to empty model

display "STATA Model 3b: Random Effect of WP Negative Mood using Grand-MC"
display "FOR COMPARISON WITH MODEL 2B ONLY BECAUSE RANDOM SLOPE IS SMUSHED"
mixed lglucAM c.TVnm0 c.PMnm0, || ID: TVnm0, covariance(un) /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog 
display "-2LL = " e(ll)*-2          // Print -2LL for model
estat recovariance, relevel(ID) correlation  // GCORR matrix
estimates store RandTV              // Save LL for LRT
lrtest RandTV FixTV                 // LRT against fixed TVnm0 model
    
display "STATA Model 3c: Fixed Effects of Sex (0=M, 1=W) by Grand-MC Negative Mood" 
mixed lglucAM c.TVnm0 c.PMnm0 c.sexMW c.TVnm0#c.sexMW c.PMnm0#c.sexMW, || ID: , /// 
      reml residuals(independent,t(day)) dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2    // Print -2LL for model
test (c.sexMW=0)(c.TVnm0#c.sexMW=0)(c.PMnm0#c.sexMW=0), small  // F-test of 3 Sex Slopes
lincom _cons*1 + c.sexMW*0, small               // Intercept: Men (at mood=0)
lincom _cons*1 + c.sexMW*1, small               // Intercept: Women (at mood=0)
lincom c.sexMW*1, small                         // Intercept: Women Diff (at mood=0)
lincom c.TVnm0*1 + c.TVnm0#c.sexMW*0, small        // L1 Within-Person Mood Effect: Men
lincom c.TVnm0*1 + c.TVnm0#c.sexMW*1, small        // L1 Within-Person Mood Effect: Women
lincom c.TVnm0#c.sexMW*1, small                    // L1 Within-Person Mood Effect: Women Diff
lincom c.TVnm0*1 + c.TVnm0#c.sexMW*0 + c.PMnm0*1 + c.PMnm0#c.sexMW*0, small    // L2 Between-Person Mood Effect: Men
lincom c.TVnm0*1 + c.TVnm0#c.sexMW*1 + c.PMnm0*1 + c.PMnm0#c.sexMW*1, small    // L2 Between-Person Mood Effect: Women
lincom c.TVnm0#c.sexMW*1 + 1*c.PMnm0#c.sexMW, small                            // L2 Between-Person Mood Effect: Women Diff
lincom c.PMnm0*1 + c.PMnm0#c.sexMW*0, small         // L2 Contextual Mood Effect: Men
lincom c.PMnm0*1 + c.PMnm0#c.sexMW*1, small         // L2 Contextual Mood Effect: Women
lincom c.PMnm0#c.sexMW*1, small                     // L2 Contextual Mood Effect: Women Diff
margins, at(c.TVnm0=(-2(1)2) c.PMnm0=(-1 1) c.sexMW=(0 1)) vsquish   // Create predicted values
marginsplot, noci xdimension(TVnm0) name(predicted_meansG, replace)  // Plot predicted values, no CI
graph export "STATA Sex Moderation Plot TVnm.png", replace
predict predsexG        // Save fixed-effect predicted outcomes
corr lglucAM predsexG   // Get total r to make R2
     display r(rho)^2   // Print total R2 relative to empty model 

 // Close external file of saved results    
 log close

